require 'digest/md5'
require 'digest/sha1'
require 'digest/sha2'

Facter.add(:dbmoduleversion, type: :aggregate) do
  has_weight 500
  # confine kernel: [ 'Linux', 'AIX' ]
  # root assume to be lib, since facter will read from lib

  fileset_monitor_wild = {

    'db2' => [
      './puppet_x/db2/*',
      './facter/db2_*',
      './scb/db2/*',
    ],
    'mongodb' => [
      './puppet_x/mongodb/*',
      './facter/mongodb_*',
      './scb/mongodb/*',
    ],
    'oracle' => [
      './puppet_x/oracle/*',
      './facter/oracle_*',
      './scb/oracle/*',
    ],
    'postgresql' => [
      './puppet_x/postgresql/*',
      './facter/postgresql_*',
      './scb/postgresql/*',
    ],

    'mssql' => [
      './puppet_x/mssql/*',
      './facter/mssql_*',
      './scb/mssql/*',
    ],

    'bbpf' => [
      './puppet_x/bigbigfacts/*',
      './facter/util/*zip*/**/*',
      './facter/util/*xz*/**/*',
      './facter/util/bigbigpuppetfacts.rb',
      './facter/bbpf_supportmatrix.rb',
    ],
    'dbprjcommon' => [
      './facter/db_inst*',
      './facter/dbmod*',
    ]
  }

  fileset_monitor = {}
  fileset_monitor_default = {
    'bbpf' => [
      './puppet_x/bigbigfacts/xzpuppetutils.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/writer.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/reader.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/libbz2.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/io.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/version.rb',
      './facter/util/bzip2-ffi-1.1.0/lib/bzip2/ffi/error.rb',
      './facter/util/rbzip2-0.3.0/lib/core_ext/io.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ffi.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/java/compressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/java/decompressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ffi/compressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ffi/errors.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ffi/decompressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ffi/constants.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/adapter.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/io.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/version.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/java.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/compressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/input_data.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/output_data.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/decompressor.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/constants.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2/ruby/crc.rb',
      './facter/util/rbzip2-0.3.0/lib/rbzip2.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/fiddle_helper.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/lib_lzma.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/stream.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/version.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/stream_writer.rb',
      './facter/util/ruby-xz-1.0.0/lib/xz/stream_reader.rb',
      './facter/util/bigbigpuppetfacts.rb',
      './facter/bbpf_supportmatrix.rb',
    ],
    'scb_mongodb_ura' => [
      './puppet_x/mongodb/process.rb',
      './puppet_x/mongodb/mongodbutils.rb',
      './puppet_x/mongodb/mongojs.rb',
      './facter/mongodb_instances.rb',
      './scb/mongodb/scb_mongodb.rb',
      './scb/mongodb/myfacter.rb',
    ],
    'scb_db2_ura' => [
      './puppet_x/db2/db2sql.rb',
      './puppet_x/db2/process.rb',
      './puppet_x/ura/process.rb',
      './facter/db2_instances_.rb',
      './facter/db2_ls.rb',
      './facter/db2_instances.rb',
      './scb/db2/scb_db2.rb',
      './scb/db2/myfacter.rb',
    ],
    'scb_postgresql_ura' => [
      './puppet_x/postgresql/process.rb',
      './puppet_x/postgresql/postgresql.rb',
      './puppet_x/ura/elif.rb',
      './puppet_x/ura/process.rb',
      './puppet_x/ura/last_login.rb',
      './facter/postgresql_instances_.rb',
      './facter/postgresql_instances.rb',
      './scb/postgresql/myfacter_origin.rb',
      './scb/postgresql/scb_postgresql.rb',
      './scb/postgresql/myfacter.rb',
    ],

  }

  fileset_monitor.merge!(fileset_monitor_default)
  fileset_monitor.merge!(fileset_monitor_wild)

  # Get-FileHash -Algorithm MD5

  fileset_monitor.each do |k, v|
    v.each do |file_to_check|
      cname = "#{k}_#{file_to_check}"
      chunk(cname.to_sym) do
        rethash = {}
        rethash[k] = {}
        file_to_check_wildcard = "#{__dir__}/../#{file_to_check}"

        # rethash[k][file_to_check_] = Facter::Core::Execution.execute("cat #{file_to_check_} | /usr/bin/md5sum  | tr -d \- ") if File.exist?(file_to_check_)

        Dir.glob(file_to_check_wildcard).each do |file_to_check_|
          next unless File.exist?(file_to_check_)
          next unless File.file?(file_to_check_)
          file_h = Digest::MD5.new
          File.open(file_to_check_, 'r') do |fh|
            while buffer = fh.read(1024) # rubocop:disable Lint/AssignmentInCondition
              file_h << buffer
            end
          end
          rethash[k][file_to_check_] = file_h.to_s
        end
        rethash
      end
    end

    #    cname = "#{k}_ALL_"
    #    chunk(cname.to_sym) do
    #      rethash = {}
    #      rethash[k] = {}
    #      rethash[k]['_ALL_'] = all_file_h.to_s
    #      rethash
    #    end
  end
end

Facter.add(:dbmoduleversion_digest) do
  has_weight 10
  setcode do
    srchash = Facter.value('dbmoduleversion')
    unless srchash.nil? || srchash.empty?
      digest = {}
      srchash.each do |k, _v|
        all_file_h = Digest::MD5.new
        all_file_h << srchash[k].to_s
        digest[k] = all_file_h.to_s
      end
      digest
    end
  end
end
